import os.path
import rox
from rox import g
import rox.tasks

class ImageMenu(g.Menu):
  def __init__(self):
    g.Menu.__init__(self)
    self._icons_updated = False
    self.connect("show", self._show)

  def _show(self, *args):
    rox.tasks.Task(self.update_icons())

  def update_icons(self, *args):
    if self._icons_updated: return
    for c in self.get_children():
      if isinstance(c, ImageMenuItem):
        c.update_icon()
        yield None
    self._icons_updated = True

class ImageMenuItem(g.ImageMenuItem):
  def __init__(self, label, **kw):
    g.ImageMenuItem.__init__(self, label)
    self.icon_size = kw.get('icon_size', g.ICON_SIZE_MENU)
    self.icon_updated = False
    if self.icon_size != g.ICON_SIZE_INVALID:
      if not kw.get('preload_icons', False) or not self.update_icon():
        if kw.get('show_missing', True):
          self.set_image(g.image_new_from_stock(
            kw.get('missing_icon', g.STOCK_FILE),
            self.icon_size))

  def get_icon_path(self):
    return None

  def update_icon(self):
    if self.icon_updated: return True
    self.icon_updated = True
    if self.icon_size == g.ICON_SIZE_INVALID:
      return True
    icon_path = self.get_icon_path()
    if not icon_path: return False
    if os.path.isabs(icon_path):
      w, h = g.icon_size_lookup(self.icon_size)
      try: pixbuf = g.gdk.pixbuf_new_from_file(icon_path)
      except: return False
      pixbuf = pixbuf.scale_simple(w, h, g.gdk.INTERP_BILINEAR)
      img = g.Image()
      img.set_from_pixbuf(pixbuf)
    else:
      img = g.image_new_from_stock(icon_path,
          self.icon_size)
    self.set_image(img)
    return True
