import os
import os.path
import glob
import rox
import rox.AppInfo
import rox.filer
from rox import g
from i18n import _
from imagemenu import ImageMenu, ImageMenuItem
from dndsource import DndSourceWidget
from ctxmenu import CtxMenuWidget

def dir_def(menu_base, directory, recursive):
  return {"menu_base":menu_base,
    "directory":directory, "recursive":recursive}

def get_menu_tree(dir_defs):
  def get_base(tree, base):
    t = tree
    if base != "":
      for c in base.split("/"):
        try: t = t[c]
        except KeyError:
          t[c] = {}; t = t[c]
    return t

  def scan_directory(tree, dir):
    for app_run in glob.glob(os.path.join(dir, "*", "AppRun")):
      if os.access(app_run, os.X_OK):
        path = os.path.dirname(app_run)
        name = os.path.basename(path)
        tree[name] = path

  def scan_directory_recursive(tree, dir):
    for d in glob.glob(os.path.join(dir, "*")):
      if not os.path.isdir(d): continue
      app_run = os.path.join(d, "AppRun")
      if os.access(app_run, os.X_OK):
        name = os.path.basename(d)
        tree[name] = d
      else:
        subtree = {}
        scan_directory_recursive(subtree, d)
        if len(subtree):
          name = os.path.basename(d)
          tree[name] = subtree

  tree = {}
  for d in dir_defs:
    base_tree = get_base(tree, d["menu_base"])
    if d["recursive"]:
      scan_directory_recursive(base_tree, d["directory"])
    else:
      scan_directory(base_tree, d["directory"])
  return tree

class RoxMenu(ImageMenu):
  def __init__(self, menu_tree, **kw):
    ImageMenu.__init__(self)
    keys = menu_tree.keys()
    keys.sort(lambda x,y: cmp(x.lower(),y.lower()))
    for name in keys:
      self.append(RoxMenuItem(menu_tree[name], name, **kw))

class RoxMenuItem(ImageMenuItem, DndSourceWidget, CtxMenuWidget):
  def __init__(self, dir, label, **kw):
    self.dir = dir
    ImageMenuItem.__init__(self, label or os.path.basename(dir), **kw)
    if type(dir) == dict:
      self.set_submenu(RoxMenu(dir, **kw))
    else:
      self.connect("activate", self.run)
      self.dnd_source_get_icon_path = self.get_icon_path
      self.dnd_source_enable()
      self.ctx_menu_enable()

  def get_icon_path(self):
    if type(self.dir) == dict:
      return g.STOCK_DIRECTORY
    for n in (".DirIcon", "AppIcon.xpm", "AppIcon.png"):
      p = os.path.join(self.dir, n)
      if os.path.exists(p): return p

  def run(self, widget, args = None):
    rox.filer.spawn_rox((self.dir,))

  def dnd_source_get_uri(self):
    return "file://" + rox.our_host_name() + rox.escape(self.dir)

  def ctx_menu_fill(self, m):
    try:
      info = rox.AppInfo.AppInfo(
          os.path.join(self.dir, "AppInfo.xml"))
    except IOError: pass
    else:
      app_run = os.path.join(self.dir, "AppRun")
      def process_app_menu(app_menu, ctx_menu):
        for e in app_menu:
          i = g.MenuItem(e["label"])
          if "option" in e:
            i.connect("activate", self.run, [e["option"]])
          if "sub-menus" in e and len(e["sub-menus"]):
            submenu = g.Menu()
            process_app_menu(e["sub-menus"], submenu)
            i.set_submenu(submenu)
          ctx_menu.append(i)
      process_app_menu(info.getAppMenu(), m)
    help = os.path.join(self.dir, "Help")
    if os.path.isdir(help):
      self.ctx_menu_append(m, g.ImageMenuItem(g.STOCK_HELP),
          lambda i: rox.filer.open_dir(help))
    if len(m.get_children()):
      m.append(g.SeparatorMenuItem())
    self.ctx_menu_append(m, g.ImageMenuItem(g.STOCK_JUMP_TO),
        lambda i: rox.filer.show_file(self.dir))
